@@ -40,13 +40,11 @@ import torchvision
 import yaml
 
 from mohou.constant import CONTINUE_FLAG_VALUE, TERMINATE_FLAG_VALUE
-from mohou.file import get_project_path
 from mohou.image_randomizer import (
     _f_randomize_depth_image,
     _f_randomize_gray_image,
     _f_randomize_rgb_image,
 )
-from mohou.setting import setting
 from mohou.utils import (
     assert_equal_with_message,
     assert_isinstance_with_message,
@@ -124,7 +122,7 @@ class ElementBase(ABC):
 
     @classmethod
     def is_concrete_type(cls):
-        return len(cls.__abstractmethods__) == 0 and len(cls.__subclasses__()) == 0
+        return len(cls.__abstractmethods__) == 0 and len(cls.__subclasses__()) == 0  # type: ignore [attr-defined]
 
     @property
     @abstractmethod
@@ -594,19 +592,45 @@ class TimeStampSequence(HasAList[float]):
             return None
 
 
-@dataclass(frozen=True)
+@dataclass
 class EpisodeData(TypeShapeTableMixin):
     sequence_dict: Dict[Type[ElementBase], ElementSequence[ElementBase]]
+    metadata: MetaData
     time_stamp_seq: Optional[TimeStampSequence] = None
-    metadata: Optional[MetaData] = None
 
     def __post_init__(self):
-        ef_seq = self.get_sequence_by_type(TerminateFlag)
-        self.check_terminate_seq(ef_seq)
+        self._validate_data()
+
+    def _validate_data(self) -> None:
+        # call this after update
+        lengths = [len(seq) for seq in self.sequence_dict.values()]
+        all_same_length = len(set(lengths)) == 1
+        assert all_same_length
+
+        # check is sequence
+        is_sequence = lengths[0] > 1
+        assert is_sequence
 
-    def __len__(self) -> int:
         # assume at least TerminateFlag is included
         assert TerminateFlag in self.sequence_dict
+        flag_seq = self.get_sequence_by_type(TerminateFlag)
+        self.check_terminate_seq(flag_seq)
+
+    @staticmethod
+    def check_terminate_seq(ef_seq: ElementSequence[TerminateFlag]):
+        # first index must be CONTINUE
+        assert ef_seq[0].numpy().item() == CONTINUE_FLAG_VALUE
+        # last index must be END
+        assert ef_seq[-1].numpy().item() == TERMINATE_FLAG_VALUE
+
+        # sequence must be like ffffffftttttt not ffffttffftttt
+        change_count = 0
+        for i in range(len(ef_seq) - 1):
+            if ef_seq[i + 1].numpy().item() != ef_seq[i].numpy().item():
+                change_count += 1
+        assert change_count == 1
+
+    def __len__(self) -> int:
         return len(self.sequence_dict[TerminateFlag])
 
     @property
@@ -623,20 +647,6 @@ class EpisodeData(TypeShapeTableMixin):
         elem_seq = ElementSequence(flag_lst)
         return elem_seq
 
-    @staticmethod
-    def check_terminate_seq(ef_seq: ElementSequence[TerminateFlag]):
-        # first index must be CONTINUE
-        assert ef_seq[0].numpy().item() == CONTINUE_FLAG_VALUE
-        # last index must be END
-        assert ef_seq[-1].numpy().item() == TERMINATE_FLAG_VALUE
-
-        # sequence must be like ffffffftttttt not ffffttffftttt
-        change_count = 0
-        for i in range(len(ef_seq) - 1):
-            if ef_seq[i + 1].numpy().item() != ef_seq[i].numpy().item():
-                change_count += 1
-        assert change_count == 1
-
     @classmethod
     def from_seq_list(
         cls,
@@ -644,12 +654,12 @@ class EpisodeData(TypeShapeTableMixin):
         timestamp_seq: Optional[TimeStampSequence] = None,
         metadata: Optional[MetaData] = None,
     ):
+        if metadata is None:
+            metadata = MetaData({})
 
         for sequence in sequence_list:
             assert isinstance(sequence, ElementSequence)
 
-        all_same_length = len(set(map(len, sequence_list))) == 1
-        assert all_same_length
         if timestamp_seq is not None:
             assert len(sequence_list[0]) == len(timestamp_seq)
 
@@ -665,7 +675,7 @@ class EpisodeData(TypeShapeTableMixin):
         assert all_different_type, "all sequences must have different type"
 
         sequence_dict = {seq.elem_type: seq for seq in sequence_list}
-        return cls(sequence_dict, timestamp_seq, metadata)  # type: ignore
+        return cls(sequence_dict, metadata, timestamp_seq)  # type: ignore
 
     def get_sequence_by_type(self, elem_type: Type[ElementT]) -> ElementSequence[ElementT]:
 
@@ -677,6 +687,14 @@ class EpisodeData(TypeShapeTableMixin):
         else:
             assert False, "element with type {} not found".format(elem_type)
 
+    def set_sequence(
+        self, elem_type: Type[PrimitiveElementT], seq: ElementSequence[PrimitiveElementT]
+    ) -> None:
+        assert issubclass(elem_type, PrimitiveElementBase)
+        assert seq.elem_type == elem_type
+        self.sequence_dict[elem_type] = seq  # type: ignore
+        self._validate_data()
+
     @overload
     def __getitem__(self, index: int) -> ElementDict:
         pass
@@ -780,9 +798,8 @@ class EpisodeData(TypeShapeTableMixin):
         if self.time_stamp_seq is not None:
             self.time_stamp_seq.dump(episode_dir_path)
 
-        if self.metadata is not None:
-            with open(episode_dir_path / "metadata.json", mode="w") as f:
-                json.dump(self.metadata, f)
+        with open(episode_dir_path / "metadata.json", mode="w") as f:
+            json.dump(self.metadata, f)
 
     @classmethod
     def load(cls, episode_dir_path: pathlib.Path) -> "EpisodeData":
@@ -790,12 +807,10 @@ class EpisodeData(TypeShapeTableMixin):
         type_seq_table = ElementSequence.load_all(episode_dir_path)
         time_stamp_seq = TimeStampSequence.load(episode_dir_path)
 
-        metadata_path = episode_dir_path / "metadata.json"
-        metadata = None
-        if metadata_path.exists():
-            with open(episode_dir_path / "metadata.json", mode="r") as f:
-                metadata = json.load(f)
-        return cls(type_seq_table, time_stamp_seq, metadata)
+        episode_dir_path / "metadata.json"
+        with open(episode_dir_path / "metadata.json", mode="r") as f:
+            metadata = MetaData(json.load(f))
+        return cls(type_seq_table, metadata, time_stamp_seq)
 
 
 @dataclass(frozen=True)
@@ -819,7 +834,7 @@ class BundleSpec(TypeShapeTableMixin):
         return cls(**d)
 
 
-_bundle_cache: Dict[Tuple[str, Optional[str]], "EpisodeBundle"] = {}  # used EpisodeBundle.load
+_bundle_cache: Dict[Tuple[Path, Optional[str]], "EpisodeBundle"] = {}  # used EpisodeBundle.load
 
 
 @dataclass
@@ -831,8 +846,8 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
 
     _episode_list: List[EpisodeData]
     _untouch_episode_list: List[EpisodeData]
-    _metadata: Optional[MetaData] = None
-    _postfix: Optional[str] = None
+    metadata: MetaData
+    postfix: Optional[str] = None
 
     def _get_has_a_list(self) -> List[EpisodeData]:
         return self._episode_list
@@ -845,23 +860,31 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
     def spec(self) -> BundleSpec:
         n_average = int(sum([len(data) for data in self._episode_list]) / len(self._episode_list))
         spec = BundleSpec(
-            len(self._episode_list) - setting.n_untouch_episode,
-            setting.n_untouch_episode,
+            len(self._episode_list),
+            len(self._untouch_episode_list),
             n_average,
             self.type_shape_table,
-            self._metadata,
+            self.metadata,
         )
         return spec
 
     @classmethod
-    def from_data_list(
+    def from_data_list(cls, *args, **kwargs) -> "EpisodeBundle":
+        """same as from_episodes. just for backward compatibility"""
+        return cls.from_episodes(*args, **kwargs)
+
+    @classmethod
+    def from_episodes(
         cls,
         data_list: List[EpisodeData],
         meta_data: Optional[MetaData] = None,
         shuffle: bool = True,
-        leave_untouch_episode: bool = True,
+        n_untouch_episode: int = 5,
     ) -> "EpisodeBundle":
 
+        if meta_data is None:
+            meta_data = MetaData({})
+
         set_types = set(functools.reduce(operator.add, [list(data.types()) for data in data_list]))
 
         n_type_appeared = len(set_types)
@@ -869,11 +892,9 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
         assert_equal_with_message(n_type_appeared, n_type_expected, "num of element type in bundle")
 
         untouch_episode_list = []
-        if leave_untouch_episode:
-            n_episode_untouch = setting.n_untouch_episode
-            assert n_episode_untouch > 0
-            interval = len(data_list) // n_episode_untouch
-            indices_untouch = [interval * i for i in range(n_episode_untouch)]
+        if n_untouch_episode > 0:
+            interval = len(data_list) // n_untouch_episode
+            indices_untouch = [interval * i for i in range(n_untouch_episode)]
             # sorted is necessary because pop changes index
             for idx in sorted(indices_untouch, reverse=True):
                 untouch_episode_list.append(data_list.pop(idx))
@@ -890,27 +911,6 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
 
         return cls(data_list, untouch_episode_list, meta_data)
 
-    @staticmethod
-    def spec_file_path(project_name: Optional[str] = None, postfix: Optional[str] = None) -> Path:
-
-        project_path = get_project_path(project_name)
-        if postfix is None:
-            yaml_file_path = project_path / "bundle_spec.yaml"
-        else:
-            yaml_file_path = project_path / "bundle_spec-{}.yaml".format(postfix)
-        return yaml_file_path
-
-    @classmethod
-    def load_spec(
-        cls, project_name: Optional[str] = None, postfix: Optional[str] = None
-    ) -> BundleSpec:
-
-        yaml_file_path = cls.spec_file_path(project_name, postfix)
-        with yaml_file_path.open(mode="r") as f:
-            d = yaml.safe_load(f)
-            spec = BundleSpec.from_dict(d)
-        return spec
-
     @classmethod
     def _load(cls, bundle_dir_path: pathlib.Path, postfix: Optional[str]) -> "EpisodeBundle":
         def load_episodes(str_startswitdth: str):
@@ -926,16 +926,16 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
         episode_list = load_episodes("episode")
         untouch_episode_list = load_episodes("untouch_episode")
 
-        metadata_file_path = bundle_dir_path / "metadata.yaml"
+        metadata_file_path = bundle_dir_path / "metadata.json"
         with metadata_file_path.open(mode="r") as f:
-            metadata = yaml.safe_load(f)
+            metadata = MetaData(json.load(f))
         bundle = EpisodeBundle(episode_list, untouch_episode_list, metadata, postfix)
         return bundle
 
     @classmethod
     def load(
         cls,
-        project_name: Optional[str] = None,
+        project_path: Path,
         postfix: Optional[str] = None,
         use_tar: bool = True,
     ) -> "EpisodeBundle":
@@ -943,11 +943,7 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
         use_tar: default True. if True, load tar archive, otherwise load from a directory
         """
 
-        if project_name is None:
-            project_name = setting.primary_project_name
-        assert isinstance(project_name, str)  # for mypy check
-
-        if (project_name, postfix) not in _bundle_cache:
+        if (project_path, postfix) not in _bundle_cache:
 
             bundle_file_without_ext = "EpisodeBundle"
             if postfix is not None:
@@ -955,7 +951,7 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
 
             if use_tar:
                 bundle_tar = bundle_file_without_ext + ".tar"
-                bundle_tar_path = get_project_path(project_name) / bundle_tar
+                bundle_tar_path = project_path / bundle_tar
 
                 with tempfile.TemporaryDirectory() as tmp_dir:
                     tmp_dir_path = pathlib.Path(tmp_dir)
@@ -967,17 +963,19 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
                     bundle = cls._load(bundle_dir_path, postfix)
 
             else:
-                bundle_dir_path = get_project_path(project_name) / bundle_file_without_ext
+                bundle_dir_path = project_path / bundle_file_without_ext
                 bundle = cls._load(bundle_dir_path, postfix)
 
-            _bundle_cache[(project_name, postfix)] = bundle
+            # because loading time of bundle is not negligible, we will cache the bundle with thep project_path and postfix key
+            # and use it when the query with the same key is asked.
+            _bundle_cache[(project_path, postfix)] = bundle
 
-        bundle = _bundle_cache[(project_name, postfix)]
+        bundle = _bundle_cache[(project_path, postfix)]
         return bundle
 
     def dump(
         self,
-        project_name: Optional[str] = None,
+        project_path: Path,
         postfix: Optional[str] = None,
         use_tar: bool = True,
     ) -> None:
@@ -991,7 +989,7 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
         Please send me a PR. Currently, no compreesion is applied.
         """
 
-        self._postfix = postfix
+        self.postfix = postfix
         bundle_file_without_ext = "EpisodeBundle" + (
             "" if postfix is None else "-{}".format(postfix)
         )
@@ -1010,13 +1008,13 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
                 episode_dir_path = bundle_dir_path / "untouch_episode{}".format(i)
                 episode.dump(episode_dir_path)
 
-            metadata_file_path = bundle_dir_path / "metadata.yaml"
+            metadata_file_path = bundle_dir_path / "metadata.json"
             with metadata_file_path.open(mode="w") as f:
-                yaml.dump(self._metadata, f, default_flow_style=False, sort_keys=False)
+                json.dump(self.metadata, f)
 
             if use_tar:
                 tarfile = bundle_file_without_ext + ".tar"
-                tarfile_full = get_project_path(project_name) / tarfile
+                tarfile_full = project_path / tarfile
                 if tarfile_full.exists():
                     os.remove(tarfile_full)
 
@@ -1024,7 +1022,6 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
                 cmd = "cd {} && tar cvf {} *".format(tmp_dir_path, tarfile_full)
                 subprocess.run(cmd, shell=True)
             else:  # just move things to project directory
-                project_path = get_project_path(project_name)
                 destination_path = project_path / bundle_file_without_ext
                 if destination_path.exists():
                     shutil.rmtree(destination_path)
@@ -1032,22 +1029,36 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
                 shutil.move(str(bundle_dir_path), str(destination_path))
 
         # extra dump just for debugging (the following info is not requried to load bundle)
-        spec_file_path = self.spec_file_path(project_name, postfix)
+        if postfix is None:
+            spec_file_path = project_path / "bundle_spec.yaml"
+        else:
+            spec_file_path = project_path / "bundle_spec-{}.yaml".format(postfix)
         with spec_file_path.open(mode="w") as f:
             yaml.dump(self.spec.to_dict(), f, default_flow_style=False, sort_keys=False)
 
     def get_untouch_bundle(self) -> "EpisodeBundle":
         """get episode bundle which is not used for training."""
-        return EpisodeBundle(self._untouch_episode_list, [], self._metadata, self._postfix)
+        return EpisodeBundle(self._untouch_episode_list, [], self.metadata, self.postfix)
 
     def get_touch_bundle(self) -> "EpisodeBundle":
         """get episode bundle which is used for training"""
-        return EpisodeBundle(self._episode_list, [], self._metadata, self._postfix)
+        return EpisodeBundle(self._episode_list, [], self.metadata, self.postfix)
+
+    def __add__(self, other: "EpisodeBundle") -> "EpisodeBundle":
+        """merge two episode bundles
+        two bundles must has same type_shape_table, i.e. each bundle have to have the same type
+        of element sequence and at the same time the shape/dimension of the each element must
+        be the same.
+        """
+        assert self.type_shape_table == other.type_shape_table
+        episode_list_new = self._episode_list + other._episode_list
+        untouch_episode_list_new = self._untouch_episode_list + other._untouch_episode_list
+        return EpisodeBundle(episode_list_new, untouch_episode_list_new, MetaData({}), None)
 
     def plot_vector_histories(
         self,
         elem_type: Type[VectorBase],
-        project_name: Optional[str] = None,
+        project_path: Path,
         hz: Optional[float] = None,
         postfix: Optional[str] = None,
     ) -> None:
@@ -1084,7 +1095,6 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
         else:
             axs[-1].set_xlabel("time [s]")
 
-        project_path = get_project_path(project_name)
         if postfix is None:
             file_path = project_path / "seq-{0}.png".format(elem_type.__name__)
         else:
